require 'rubygems'
require 'json'
require_relative 'POSOMetaclasses'
require_relative 'POSOTypes'
require_relative 'usi_metaclasses'

def make_definition_set
	# File Paths
	useful_root = '../Useful'
	model_user_source = File.join(useful_root, 'Shared', 'Models')
	model_generated_source = File.join(model_user_source, 'Generated')

	test_root = '../UsefulTests'
	model_test = File.join(test_root, 'Shared', 'Models')
	model_generated_test = File.join(model_test, 'Generated')

	# Standard USI Model settings
	uis_model_characteristics = [:json_serializable, :comparable, :make_test_class, :create_user_class]

	# USI Model (Wire Data) settings
	defs = SwiftDefinitionSet.new( generated_root:model_generated_source,
								   generated_user_root:model_user_source,
								   generated_test_root:model_generated_test)

	base = SwiftClass.new(defs, 'USIBaseModel', ['NSObject'], characteristics: uis_model_characteristics )
	base.add_simple_class_property( "restEndpoint", :AnvilEndpoint, value:".None", override:false )

	SwiftPersistentProperty.new( base, '__v',             :String, 	:optional, rest_omit: :omit_if_null)
	SwiftPersistentProperty.new( base, '_id',             :String, 	:optional, rest_omit: :omit_if_null )
	SwiftPersistentProperty.new( base, 'organizationID',  :String, 	:optional, json_key:'organizationId', rest_omit: :omit   )
	SwiftPersistentProperty.new( base, 'updatedAt',       :NSDate, 	:optional, rest_omit: :omit )

	address = USIWireDataClass.new(defs, 'Address', endpoint:".Addresses"  )
	SwiftPersistentProperty.new( address, 'buildingName',		:String,	:optional )
	SwiftPersistentProperty.new( address, 'city',				:String,	:optional )
	SwiftPersistentProperty.new( address, 'country',			:String,	:optional )
	SwiftPersistentProperty.new( address, 'governingDistrict',	:String,	:optional )
	SwiftPersistentProperty.new( address, 'latitude',			:Double,	:optional )
	SwiftPersistentProperty.new( address, 'localMunicipality',	:String,	:optional )
	SwiftPersistentProperty.new( address, 'longitude',			:Double,	:optional )
	SwiftPersistentProperty.new( address, 'postalArea',			:String,	:optional )
	SwiftPersistentProperty.new( address, 'streetDirection',	:String,	:optional )
	SwiftPersistentProperty.new( address, 'streetName',			:String,	:optional )
	SwiftPersistentProperty.new( address, 'streetNumber',		:Int,		:optional )
	SwiftPersistentProperty.new( address, 'streetNumberSuffix',	:String,	:optional )
	SwiftPersistentProperty.new( address, 'streetType',			:String,	:optional )
	SwiftPersistentProperty.new( address, 'title',				:String,	:optional )
	SwiftPersistentProperty.new( address, 'type',				:String,	:optional )
	SwiftPersistentProperty.new( address, 'typeIdentifier',		:String,	:optional )

	affOrg = USIWireDataClass.new( defs, 'AffiliateOrganization', endpoint:".AffiliateOrganizations" )
	SwiftPersistentProperty.new( affOrg, 'activeCustomer', 		:Bool,			:optional )
	SwiftPersistentProperty.new( affOrg, 'activePartner',		:Bool,			:optional )
	SwiftPersistentProperty.new( affOrg, 'customer',			:Bool,			:optional )
	SwiftPersistentProperty.new( affOrg, 'otherOrganization',	:USIObjectID,	:optional )
	SwiftPersistentProperty.new( affOrg, 'partner', 			:Bool,			:optional )

	contact = USIWireDataClass.new( defs, 'Contact', endpoint:".Contacts"  )
	SwiftPersistentProperty.new( contact, 'email', 			:String, 		:optional )
	SwiftPersistentProperty.new( contact, 'firstName', 		:String, 		:optional )
	SwiftPersistentProperty.new( contact, 'lastName', 		:String, 		:optional )
	SwiftPersistentProperty.new( contact, 'phoneNumber', 	:PhoneNumber, 	:optional )

	customer = USIWireDataClass.new( defs, 'Customer', endpoint:".Customers" )
	SwiftPersistentProperty.new( customer, 'name', 			:String, 		:optional )
	SwiftPersistentProperty.new( customer, 'email', 		:String, 		:optional )
	SwiftPersistentProperty.new( customer, 'types', 		:String, 		:optional, collection_type: :array )
	SwiftPersistentProperty.new( customer, 'account', 		:String, 		:optional )
	SwiftPersistentProperty.new( customer, 'addresses', 	:Address,		:optional, collection_type: :array )
	SwiftPersistentProperty.new( customer, 'contacts', 		:Contact,		:optional, collection_type: :array  )
	SwiftPersistentProperty.new( customer, 'phoneNumbers', 	:PhoneNumber, 	:optional, collection_type: :array )

	customerSummary = USIWireDataClass.new( defs, 'CustomerSummary' )
	SwiftPersistentProperty.new( customerSummary, 'name', 		:String, 		:optional )
	SwiftPersistentProperty.new( customerSummary, 'email', 		:String, 		:optional )
	SwiftPersistentProperty.new( customerSummary, 'types', 		:String, 		:optional, collection_type: :array  )
	SwiftPersistentProperty.new( customerSummary, 'account', 	:String, 		:optional )

	detail = USIWireDataClass.new( defs, 'Detail', endpoint:".Details" )
	SwiftPersistentProperty.new( detail, 'detailText', 	:String,	:optional )
	SwiftPersistentProperty.new( detail, 'userId', 		:String,	:optional )
	SwiftPersistentProperty.new( detail, 'workFormID', 	:String,	:optional, json_key:'workFormid'  )
	SwiftPersistentProperty.new( detail, 'title', 		:String,	:optional )

	materialType = USIWireDataClass.new( defs, 'MaterialType', endpoint:".MaterialTypes" )
	SwiftPersistentProperty.new( materialType, 'materialDescription', 	:String,	:optional, json_key:'description' )
	SwiftPersistentProperty.new( materialType, 'defaultBillablePrice', 	:Int,		:optional )
	SwiftPersistentProperty.new( materialType, 'defaultCost', 			:Int,		:optional )
	SwiftPersistentProperty.new( materialType, 'name',					:String,	:optional )
	SwiftPersistentProperty.new( materialType, 'partNumber',			:String,	:optional )
	SwiftPersistentProperty.new( materialType, 'unit',					:String,	:optional )

	material = USIWireDataClass.new(defs, 'Material', endpoint:".Materials")
	SwiftPersistentProperty.new( material, 'workFormID', 			:String,	:optional )
	SwiftPersistentProperty.new( material, 'materialTypeID', 		:String,	:optional )
	SwiftPersistentProperty.new( material, 'materialDescription', 	:String,	:optional, json_key:'description' )
	SwiftPersistentProperty.new( material, 'billablePrice', 		:Int,		:optional )
	SwiftPersistentProperty.new( material, 'cost', 					:Int,		:optional )
	SwiftPersistentProperty.new( material, 'name',					:String,	:optional )
	SwiftPersistentProperty.new( material, 'partNumber',			:String,	:optional )
	SwiftPersistentProperty.new( material, 'unit',					:String,	:optional )

	org = USIWireDataClass.new( defs, 'Organization', endpoint:".Organizations" )
	SwiftPersistentProperty.new( org, 'addresses',					:Address, 					:optional, collection_type: :array )
	SwiftPersistentProperty.new( org, 'affiliateOrganizations', 	:Organization, 	:optional, collection_type: :array )
	SwiftPersistentProperty.new( org, 'contacts',					:Contact, 					:optional, collection_type: :array )
	SwiftPersistentProperty.new( org, 'defaultTaxRate',				:Int,						:optional )
	SwiftPersistentProperty.new( org, 'email',						:String,					:optional )
	SwiftPersistentProperty.new( org, 'logoURIs',					:String, 					:optional, collection_type: :array )
	SwiftPersistentProperty.new( org, 'materialTypes', 				:MaterialType, 				:optional, collection_type: :array )
	SwiftPersistentProperty.new( org, 'name',						:String,					:optional )
	SwiftPersistentProperty.new( org, 'phoneNumbers',				:PhoneNumber,				:optional, collection_type: :array )
	SwiftPersistentProperty.new( org, 'serviceTypes',				:ServiceType, 				:optional, collection_type: :array )
	# 'subscription (ref to Subscription)
	SwiftPersistentProperty.new( org, 'timezone',					:String,					:optional )
	SwiftPersistentProperty.new( org, 'website',					:String,					:optional )
	SwiftPersistentProperty.new( org, 'users',						:User,						:optional, collection_type: :array )

	payRate = USIWireDataClass.new(defs, 'PayRate', endpoint:".PayRates" )
	SwiftPersistentProperty.new( payRate, 'rate', 		:Int, 		:optional )
	SwiftPersistentProperty.new( payRate, 'taxType', 	:TaxType, 	:optional )

	permission = SwiftEnum.new(defs, 'Permission', ['String'] )
	SwiftEnumCase.new( permission, 'Admin', 		'ADMIN' )

	phoneNumber = USIWireDataClass.new(defs, 'PhoneNumber', endpoint:".PhoneNumbers" )
	SwiftPersistentProperty.new( phoneNumber, 'areaCode', 		:Int, 		:optional )
	SwiftPersistentProperty.new( phoneNumber, 'countryCode',	:Int, 		:optional )
	SwiftPersistentProperty.new( phoneNumber, 'exchange',		:Int, 		:optional )
	SwiftPersistentProperty.new( phoneNumber, 'line',			:Int, 		:optional )
	SwiftPersistentProperty.new( phoneNumber, 'type',			:String,	:optional )

	role = USIWireDataClass.new( defs, 'Role', endpoint:".Roles" )
	SwiftPersistentProperty.new( role, 'permissions', 	:Permission, 	:optional, collection_type: :array )

	serviceType = USIWireDataClass.new( defs, 'ServiceType', endpoint:".ServiceTypes" )
	SwiftPersistentProperty.new( serviceType, 'defaultBillableRate',	:Int, 		:optional )
	SwiftPersistentProperty.new( serviceType, 'serviceDescription',		:String, 	:optional, json_key:'description')
	SwiftPersistentProperty.new( serviceType, 'isHourlyRate',			:Bool,		:optional )
	SwiftPersistentProperty.new( serviceType, 'name',					:String,	:optional )

	service = USIWireDataClass.new( defs, 'Service', endpoint: ".Services" )
	SwiftPersistentProperty.new( service, 'billableRate',			:Int, 		:optional )
	SwiftPersistentProperty.new( service, 'serviceDescription',		:String, 	:optional, json_key:'description')
	SwiftPersistentProperty.new( service, 'isHourlyRate',			:Bool,		:optional )
	SwiftPersistentProperty.new( service, 'name',					:String,	:optional )
	SwiftPersistentProperty.new( service, 'serviceTypeId',			:String,	:optional )
	SwiftPersistentProperty.new( service, 'workFormId',				:String,	:optional )

	tax_type = SwiftEnum.new(defs, 'TaxType', ['String']  )
	SwiftEnumCase.new( tax_type, 'HourlyEmployee', 		'HOURLY_EMPLOYEE' )
	SwiftEnumCase.new( tax_type, 'None', 				'NONE' )
	SwiftEnumCase.new( tax_type, 'SalariedContractor', 	'SALARIED_CONTRACTOR' )
	SwiftEnumCase.new( tax_type, 'SalariedEmployee', 	'SALARIED_EMPLOYEE' )

	user = USIWireDataClass.new(defs, 'User', endpoint:".Users")
	SwiftPersistentProperty.new( user, 'addresses',					:Address, 		:optional, collection_type: :array )
	SwiftPersistentProperty.new( user, 'disabled',					:Bool, 			:optional )
	SwiftPersistentProperty.new( user, 'emergencyContacts',			:Contact, 		:optional, collection_type: :array )
	SwiftPersistentProperty.new( user, 'email',						:String, 		:optional )
	SwiftPersistentProperty.new( user, 'firstName',					:String, 		:optional )
	SwiftPersistentProperty.new( user, 'lastName',					:String, 		:optional )
	SwiftPersistentProperty.new( user, 'latitude',					:Double, 		:optional )
	SwiftPersistentProperty.new( user, 'longitude',					:Double, 		:optional )
	SwiftPersistentProperty.new( user, 'passwordHash',				:String, 		:optional )
	SwiftPersistentProperty.new( user, 'payRate',					:PayRate, 		:optional )
	SwiftPersistentProperty.new( user, 'phoneNumbers',				:PhoneNumber,	:optional, collection_type: :array )
	SwiftPersistentProperty.new( user, 'photoURIs',					:String, 		:optional, collection_type: :array )
	SwiftPersistentProperty.new( user, 'role',						:Role, 			:optional )

	userSummary = USIWireDataClass.new( defs, 'UserSummary' )
	SwiftPersistentProperty.new( userSummary, 'isDisabled',			:Bool, 			:optional )
	SwiftPersistentProperty.new( userSummary, 'email',				:String, 		:optional )
	SwiftPersistentProperty.new( userSummary, 'firstName',			:String, 		:optional )
	SwiftPersistentProperty.new( userSummary, 'lastName',			:String, 		:optional )
	SwiftPersistentProperty.new( userSummary, 'nickname',			:String, 		:optional )

	workForm = USIWireDataClass.new( defs, 'WorkForm', endpoint:".WorkForms" )
	SwiftPersistentProperty.new( workForm, 'customer',				:CustomerSummary,	:optional )
	SwiftPersistentProperty.new( workForm, 'formDescription',		:String,			:optional, json_key:'description' )
	SwiftPersistentProperty.new( workForm, 'assignedToUser', 		:UserSummary,		:optional )
	SwiftPersistentProperty.new( workForm, 'createdByUser', 		:UserSummary,		:optional )
	SwiftPersistentProperty.new( workForm, 'details',				:Detail,			:optional, collection_type: :array )
	SwiftPersistentProperty.new( workForm, 'materials',				:Material,			:optional, collection_type: :array )
	SwiftPersistentProperty.new( workForm, 'services',				:Service,			:optional, collection_type: :array )
	SwiftPersistentProperty.new( workForm, 'crew', 					:UserSummary,		:optional, collection_type: :array  )
	SwiftPersistentProperty.new( workForm, 'formId',				:String,			:optional )
	SwiftPersistentProperty.new( workForm, 'formType',				:WorkFormType,		:optional )
	SwiftPersistentProperty.new( workForm, 'workFlowId',			:String,			:optional )
	SwiftPersistentProperty.new( workForm, 'previousFormId',		:String,			:optional )
	SwiftPersistentProperty.new( workForm, 'status',				:WorkStatusType,	:optional )
	SwiftPersistentProperty.new( workForm, 'startDate',				:NSDate,			:optional )
	SwiftPersistentProperty.new( workForm, 'endDate',				:NSDate,			:optional )
	SwiftPersistentProperty.new( workForm, 'marqueeImageURL',		:String,			:optional )
	SwiftPersistentProperty.new( workForm, 'totalValue',			:Int,				:optional )

	work_form_type = SwiftEnum.new(defs, 'WorkFormType', ['String']  )
	SwiftEnumCase.new( work_form_type, 'Request', 		'REQUEST' )
	SwiftEnumCase.new( work_form_type, 'Inspection', 	'INSPECTION' )
	SwiftEnumCase.new( work_form_type, 'Estimate', 		'ESTIMATE' )
	SwiftEnumCase.new( work_form_type, 'WorkOrder', 	'WORKORDER' )
	SwiftEnumCase.new( work_form_type, 'Invoice', 		'INVOICE' )

	work_status_type = SwiftEnum.new(defs, 'WorkStatusType', ['String']  )
	SwiftEnumCase.new( work_status_type, 'Draft', 			'DRAFT' )
	# SwiftEnumCase.new( work_status_type, 'Unscheduled', 	'UNSCHEDULED' )
	SwiftEnumCase.new( work_status_type, 'Assigned', 		'ASSIGNED' )
	SwiftEnumCase.new( work_status_type, 'Scheduled', 		'SCHEDULED' )
	SwiftEnumCase.new( work_status_type, 'Dispatched', 		'DISPATCHED' )
	SwiftEnumCase.new( work_status_type, 'InProgress', 		'IN-PROGRESS' )
	SwiftEnumCase.new( work_status_type, 'Completed', 		'COMPLETED' )
	SwiftEnumCase.new( work_status_type, 'Rejected', 		'REJECTED' )
	SwiftEnumCase.new( work_status_type, 'Submitted', 		'SUBMITTED' )

	return defs
end
